/************************************************************************/
/*				                        		*/
/*	RDKBasic_Motor                 		                        */
/*						                  	*/
/*					                        	*/
/************************************************************************/
/*	Author: Jordan Retz						*/
/*	Copyright 2012, Digilent Inc.					*/
/************************************************************************/
/*  Module Description: 			             		*/
/*					                        	*/
/* This module implements a simple demo application that demonstrates   */
/* how to program the RDK Basic - Motor.      		                */
/*									*/
/*	Functionality:							*/
/*									*/
/* The application uses the onboard buttons to control two motors. When */
/* button BTN1 is pressed, the robot will move forwardspecified below:  */
/*                                                                      */
/*  Instruction Code    BTN Pressed        Function                     */
/*        0x0            None              Stop                         */
/*        0x1            BTN1              Stop                         */
/*        0x2            BTN2              Move Forward                 */
/*                                                                      */
/* When the BTN2 is pressed, the robot will begin moving. If BTN1 is    */
/* pressed, the robot will stop.                                        */
/*                                                                      */
/* A PmodHB5 must be connected to the top row of header JD for the right*/
/* motor, and a PmodHB5 must be connected to the bottom row of header   */
/* JD for the left motor.                                               */
/*                                                                      */
/*     Programming:                                                     */
/*                                                                      */
/* Make sure that the MX4cK is plugged into the computer via USB cable, */
/* and that the board is powered.                                       */
/*                                                                      */
/* From the toolbar above, select "Tools->Board->MX4cK". From the Start */
/* menu, navitgate to "Start->Control Panel->Hardware and Sound->Device */
/* Manager". Select "Ports", and take note of what COM port (COMx) is   */
/* labeled "USB Serial Port". If you are unsure, unplug the MX4cK from  */
/* the computer and watch which COM port disappears, and then           */
/* reappears, when you plug the MX4cK back into the computer.           */
/* From the toolbar above, select "Tools->Serial Port" and then the COM */
/* port that you just found. Apply power to the MX4cK and click on the  */
/* "Upload" button above.                                               */
/*                                                                      */
/************************************************************************/
/*  Revision History:			        			*/
/*					                        	*/
/*	6/22/2012 (JordanR): created		       			*/
/*      8/22/2012 (JordanR): modified to use analogWrite to drive motors*/
/*						                     	*/
/************************************************************************/

/* ------------------------------------------------------------ */
/*                Include File Definitions                      */
/* ------------------------------------------------------------ */
#include <plib.h>

/* ------------------------------------------------------------ */
/*				Local Type Definitions		*/
/* ------------------------------------------------------------ */
/* These definitions apply if the Right Motor is connected to a PmodHB5 
** on the top row of JD, and the Left Motor is connected to a PmodHB5
** on the bottom row of JD.
*/
#define    pinLeftEn    29
#define    pinLeftDir   28
#define    pinRightEn   25
#define    pinRightDir  24

/* These definitions apply for MX4cK Rev B
*/
#define    pinBTN1      42
#define    pinBTN2      43

/* ------------------------------------------------------------ */
/*                Global Variables                              */
/* ------------------------------------------------------------ */
byte       bPrevAction;
byte       bBtn1;
byte       bBtn2;
byte       bRDKEnable;

/* ------------------------------------------------------------ */
/*                Forward Declarations                          */
/* ------------------------------------------------------------ */
void CheckButtons();
void StopRobot();
void Forward();
void Backward();
void TurnRight();
void TurnLeft();


/* ------------------------------------------------------------ */
/*                Procedure Definitions                         */
/* ------------------------------------------------------------ */

/***  void setup()
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Initialize the system.
*/
void setup() {
  /*Setup Input/Output on Pins
  */
  pinMode(pinRightDir, OUTPUT);
  pinMode(pinRightEn, OUTPUT);
  pinMode(pinLeftDir, OUTPUT);
  pinMode(pinLeftEn, OUTPUT);                
  
  pinMode(pinBTN1, INPUT);
  pinMode(pinBTN2, INPUT);
  
  /* Disable Motors
  */
  digitalWrite(pinLeftEn, LOW);
  digitalWrite(pinRightEn, LOW);
  
  /* Misc.
  */
  StopRobot();
  bPrevAction = 0x00;
}

/* ------------------------------------------------------------ */
/***  void loop();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Main application loop.
*/
void loop() {
  CheckButtons();
  
  if(bRDKEnable == 0x2 && bRDKEnable != bPrevAction) { // move forward
    
    Forward();
    
    bPrevAction = bRDKEnable;
  }
  else if(bRDKEnable == 0x1) {
    
    StopRobot();
    
    bPrevAction = bRDKEnable;
  }
}

/* ------------------------------------------------------------ */
/***  void CheckButtons();
**
**  Parameters:
**    none
**
**  Return Value:
**  none
**
**  Errors:
**  none
**
**  Description:
**    Fetches the state of the buttons. 
*/
void CheckButtons() { 
    unsigned int i = 0;
    
    bBtn1 = 0;
    bBtn2 = 0;
  
    for(i = 0; i < 8; i++) {
      bBtn1 |= digitalRead(pinBTN1) << i;
      bBtn2 |= digitalRead(pinBTN2) << i;
      delay(1);
    }    
    
    if(bBtn2 & 0xFF) {
      bRDKEnable = 0x2;
    }
    else if(bBtn1 & 0xFF) {
      bRDKEnable = 0x1;
    }
    else {
      //continue previous instruction
    }
}

/* ------------------------------------------------------------ */
/***	void StopRobot();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Stops robot by finding Servo position and reducing or increasing to 90. 
**    Then detaches pins.
*/
void StopRobot() {
  
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
}

/* ------------------------------------------------------------ */
/***	void Forward();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Moves robot forward
*/
void Forward() {
   
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
    digitalWrite(pinRightDir, LOW);
    digitalWrite(pinLeftDir, HIGH);
    
    analogWrite(pinLeftEn, 127);
    analogWrite(pinRightEn, 127);

}

/* ------------------------------------------------------------ */
/***	void Backward();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Moves robot backward
*/
void Backward() {
  
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
    digitalWrite(pinRightDir, HIGH);
    digitalWrite(pinLeftDir, LOW);
    
    analogWrite(pinLeftEn, 127);
    analogWrite(pinRightEn, 127);

  
}

/* ------------------------------------------------------------ */
/***	void TurnRight();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Turns robot right
*/
void TurnRight() {  
  
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
    digitalWrite(pinRightDir, HIGH);
    digitalWrite(pinLeftDir, HIGH);
    
    analogWrite(pinLeftEn, 127);
    analogWrite(pinRightEn, 127);
    
}

/* ------------------------------------------------------------ */
/***	void TurnLeft();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**   Turns robot left
*/
void TurnLeft() {  
    
    /* Always disable the motors before changing their direction
    */
    analogWrite(pinLeftEn, 0);
    analogWrite(pinRightEn, 0);
    
    digitalWrite(pinRightDir, LOW);
    digitalWrite(pinLeftDir,LOW);
    
    analogWrite(pinLeftEn, 127);
    analogWrite(pinRightEn, 127);
}
